#version 440

layout(points) in;
layout(points) out;
layout(max_vertices = 40) out;

// All that we get from vertex shader

in vec3 vPositionPass[];
in vec3 vVelocityPass[];
in vec3 vColorPass[];
in float fLifeTimePass[];
in float fSizePass[];
in float iUpdateTextureMillisPass[];
in float iCurrentAnimTimeMillisPass[];
in int iTypePass[];
in int iTexPosPass[];

// All that we send further

out vec3 vPositionOut;
out vec3 vVelocityOut;
out vec3 vColorOut;
out float fLifeTimeOut;
out float fSizeOut;
out float iUpdateTextureMillisOut;
out float iCurrentAnimTimeMillisOut;
out int iTypeOut;
out int iTexPosOut;

uniform vec3 vGenPosition; // Position where new particles are spawned
uniform vec3 vGenGravityVector; 
uniform vec3 vGenVelocityMin;
uniform vec3 vGenVelocityRange;

uniform vec3 vGenColor;
uniform float fGenSize; 

uniform float fGenLifeMin, fGenLifeRange; 
uniform float fTimePassed; 

uniform vec3 vRandomSeed; 
vec3 vLocalSeed;

uniform int iNumToGenerate; 
uniform int rowss;
uniform int cols;

// Random generator (0...1)
float randZeroOne()
{
    uint n = floatBitsToUint(vLocalSeed.y * 214013.0 + vLocalSeed.x * 2531011.0 + vLocalSeed.z * 141251.0);
    n = n * (n * n * 15731u + 789221u);
    n = (n >> 9u) | 0x3F800000u;
 
    float fRes =  2.0 - uintBitsToFloat(n);
    vLocalSeed = vec3(vLocalSeed.x + 147158.0 * fRes, vLocalSeed.y*fRes  + 415161.0 * fRes, vLocalSeed.z + 324154.0*fRes);
    return fRes;
}

void main()
{
	gl_Position = vec4(0);
	iUpdateTextureMillisOut = iUpdateTextureMillisPass[0];
	iCurrentAnimTimeMillisOut = iCurrentAnimTimeMillisPass[0];
	iTexPosOut = iTexPosPass[0];

  vLocalSeed = vRandomSeed;

  vPositionOut = vPositionPass[0];
  vVelocityOut = vVelocityPass[0];
  if(iTypePass[0] != 0)vPositionOut += vVelocityOut*fTimePassed;
  if(iTypePass[0] != 0)vVelocityOut += vGenGravityVector*fTimePassed;

  vColorOut = vColorPass[0];
  fLifeTimeOut = fLifeTimePass[0]-fTimePassed;
  iCurrentAnimTimeMillisOut += fTimePassed;

  fSizeOut = fSizePass[0];
  iTypeOut = iTypePass[0];
    
  if(iTypeOut == 0)
  {
    EmitVertex();
    EndPrimitive();
    
    for(int i = 0; i < iNumToGenerate; i++)
    {
      vPositionOut = vGenPosition;
      vVelocityOut = vGenVelocityMin+vec3(vGenVelocityRange.x*randZeroOne(), vGenVelocityRange.y*randZeroOne(), vGenVelocityRange.z*randZeroOne());
      vColorOut = vGenColor;
      fLifeTimeOut = fGenLifeMin+fGenLifeRange*randZeroOne();
      fSizeOut = fGenSize * randZeroOne();
      iTypeOut = 1;
	  iUpdateTextureMillisOut = fGenLifeRange/float(rowss * cols)*0.5;
	  iCurrentAnimTimeMillisOut = 0;
	  iTexPosOut = 0;
      EmitVertex();
      EndPrimitive();
    }
  }else if(fLifeTimePass[0] > 0.0){
	if ( iCurrentAnimTimeMillisPass[0] >= iUpdateTextureMillisPass[0] && (rowss * cols) > 1) {
        iCurrentAnimTimeMillisOut = 0;
		int texPos = iTexPosPass[0];
		texPos++; 
        if (texPos < (rowss * cols)) {
           iTexPosOut = texPos;
        } else {
            iTexPosOut = 0;
		}
		  
	}
	//iTexPosOut = iTexPosPass[0];
	EmitVertex();
	EndPrimitive(); 	  
  }
}